@@ -361,6 +361,11 @@ namespace Nes
 			return 0;
 		}
 
+		uint Apu::GetCtrl()
+		{
+			return ctrl;
+		}
+
 		Result Apu::SetSpeed(const uint speed)
 		{
 			if (settings.speed == speed)
@@ -1670,7 +1675,7 @@ namespace Nes
 			step = 0x7;
 			status = STATUS_COUNTING;
 			waveLength = 0;
-			linearCtrl = 0;
+			//linearCtrl = 0;
 			linearCounter = 0;
 
 			lengthCounter.Reset();
@@ -1870,7 +1875,7 @@ namespace Nes
 					amp = (sum * outputVolume + rate/2) / rate * 3;
 				}
 			}
-			else if (amp < Channel::OUTPUT_DECAY)
+			/*else if (amp < Channel::OUTPUT_DECAY)
 			{
 				return 0;
 			}
@@ -1878,7 +1883,7 @@ namespace Nes
 			{
 				amp -= Channel::OUTPUT_DECAY;
 				step &= STEP_CHECK;
-			}
+			}*/
 
 			return amp;
 		}
@@ -2120,7 +2125,6 @@ namespace Nes
 			dma.buffered       = false;
 			dma.address        = 0xC000;
 			dma.buffer         = 0x00;
-			overclockingIsSafe = true;
 		}
 
 		Cycle Apu::Dmc::GetResetFrequency(CpuModel model)
@@ -2320,9 +2324,15 @@ namespace Nes
 
 				cpu.StealCycles( cpu.GetClock(1) );
 
-				// This is disabled until a real solution is discovered
-				//if ((readAddress & 0xF000) != 0x4000)
-				//	cpu.Peek( readAddress );
+				/* According to dmc_dma_during_read4/dma_2007_read, DMC DMA during read causes
+				 * 2-3 extra $2007 reads before the real read. The nesdev wiki states that this
+				 * also happens when polling $2002 for vblank.
+				*/
+				if ((readAddress & 0xF000) != 0x4000)
+				{
+					cpu.Peek( readAddress );
+					cpu.Peek( readAddress );
+				}
 
 				cpu.StealCycles( cpu.GetClock(1) );
 				cpu.Peek( readAddress );
@@ -2361,31 +2371,16 @@ namespace Nes
 		{
 			out.dac = data & 0x7F;
 			curSample = out.dac * outputVolume;
-
-			if (out.dac != 0)
-			{
-				overclockingIsSafe = false;
-			}
 		}
 
 		NST_SINGLE_CALL void Apu::Dmc::WriteReg2(const uint data)
 		{
 			regs.address = 0xC000 | (data << 6);
-
-			if (regs.address != 0)
-			{
-				overclockingIsSafe = true;
-			}
 		}
 
 		NST_SINGLE_CALL void Apu::Dmc::WriteReg3(const uint data)
 		{
 			regs.lengthCounter = (data << 4) + 1;
-
-			if (regs.lengthCounter != 0)
-			{
-				overclockingIsSafe = true;
-			}
 		}
 
 		NST_SINGLE_CALL bool Apu::Dmc::ClockDAC()
