@@ -373,7 +373,7 @@ namespace Nes
 					{
 						State::Loader::Data<4> data( state );
 
-						io.ctrl = data[0];
+						io.ctrl = adapter.ctrl = data[0];
 						io.port = data[1];
 						break;
 					}
@@ -550,7 +550,13 @@ namespace Nes
 
 		NES_POKE_D(Fds,4023)
 		{
-			io.ctrl = data;
+			io.ctrl = adapter.ctrl = data;
+
+			if (!(io.ctrl & Io::CTRL0_DISK_ENABLED))
+			{
+				cpu.ClearIRQ();
+				adapter.DisableIRQ();
+			}
 		}
 
 		NES_POKE_D(Fds,4026)
@@ -783,21 +789,25 @@ namespace Nes
 		#pragma optimize("", on)
 		#endif
 
-		void Fds::Unit::Timer::Advance(uint& timer)
+		bool Fds::Unit::Timer::Clock()
 		{
-			timer |= STATUS_PENDING_IRQ;
+			bool retval = false;
 
-			if (ctrl & CTRL_REPEAT)
-				count = latch;
-			else
-				ctrl &= ~uint(CTRL_ENABLED);
-			
-			latch = 0; // Fixes Kaettekita Mario Bros - FHorse/dragon2snow
-		}
+			if (ctrl & CTRL_ENABLED)
+			{
+				if (count == 0)
+				{
+					retval = true;
+					count = latch;
 
-		NST_SINGLE_CALL bool Fds::Unit::Timer::Clock()
-		{
-			return !(ctrl & CTRL_ENABLED) || !count || --count;
+					if (!(ctrl & CTRL_REPEAT))
+						ctrl &= ~uint(CTRL_ENABLED);
+				}
+				else
+					count--;
+			}
+
+			return retval;
 		}
 
 		#ifdef NST_MSVC_OPTIMIZE
@@ -1202,11 +1212,15 @@ namespace Nes
 
 		ibool Fds::Unit::Clock()
 		{
-			return
-			(
-				(timer.Clock() ? 0 : (timer.Advance(status), 1)) |
-				(drive.Clock() ? 0 : drive.Advance(status))
-			);
+			bool retval = false;
+
+			if (timer.Clock())
+			{
+				status |= STATUS_PENDING_IRQ;
+				retval = true;
+			}
+
+			return (retval | (drive.Clock() ? 0 : drive.Advance(status)));
 		}
 
 		#ifdef NST_MSVC_OPTIMIZE
@@ -1216,6 +1230,12 @@ namespace Nes
 		Fds::Adapter::Adapter(Cpu& c,const Disks::Sides& s)
 		: Timer::M2<Unit>(c,s) {}
 
+		void Fds::Adapter::DisableIRQ()
+		{
+			unit.status &= ~uint(Unit::STATUS_PENDING_IRQ);
+			unit.timer.ctrl &= ~uint(Unit::Timer::CTRL_ENABLED);
+		}
+
 		void Fds::Adapter::Reset(Cpu& cpu,byte* const io,bool protect)
 		{
 			Timer::M2<Unit>::Reset( true, true );
@@ -1379,12 +1399,17 @@ namespace Nes
 		{
 			Update();
 
+			if (!(ctrl & Io::CTRL0_DISK_ENABLED))
+				return;
+
 			unit.timer.ctrl = data;
 			unit.timer.count = unit.timer.latch;
 			unit.status &= Unit::STATUS_TRANSFERED;
 
-			if (!unit.status)
-				ClearIRQ();
+			if (data & Unit::Timer::CTRL_ENABLED)
+				return;
+
+			ClearIRQ();
 		}
 
 		NES_POKE_D(Fds::Adapter,4024)
